/***********************************************************************
 * IBM Confidential 
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2006
 *
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 ************************************************************************/

package com.ibm.cds;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;

import org.eclipse.osgi.baseadaptor.bundlefile.BundleEntry;
import org.eclipse.osgi.baseadaptor.bundlefile.BundleFile;

import com.ibm.oti.shared.SharedClassURLHelper;

/**
 * Wraps an actual BundleFile object for purposes of loading classes from the
 * shared classes cache. 
 */
public class CDSBundleFile extends BundleFile {
	private BundleFile wrapped; // the real bundle file
	private URL url; // the URL to the content of the real bundle file
	private SharedClassURLHelper urlHelper; // the url helper set by the classloader
	private boolean primed = false;

	/**
	 * The constructor
	 * @param wrapped the real bundle file
	 */
	public CDSBundleFile(BundleFile wrapped) {
		super(wrapped.getBaseFile());
		this.wrapped = wrapped;
		// get the url to the content of the real bundle file
		try {
			this.url = new URL("file", "", wrapped.getBaseFile().getAbsolutePath());
		} catch (MalformedURLException e) {
			// do nothing
		}
	}

	public CDSBundleFile(BundleFile bundleFile, SharedClassURLHelper urlHelper) {
		this(bundleFile);
		this.urlHelper = urlHelper;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.osgi.baseadaptor.bundlefile.BundleFile#close()
	 * 
	 * just closes the wrapped bundle file
	 */
	public void close() throws IOException {
		wrapped.close();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.osgi.baseadaptor.bundlefile.BundleFile#containsDir(java.lang.String)
	 * 
	 * just uses the wrapped bundle file to answer the call
	 */
	public boolean containsDir(String dir) {
		return wrapped.containsDir(dir);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.osgi.baseadaptor.bundlefile.BundleFile#getEntry(java.lang.String)
	 * 
	 * If the path is for a class then it will look for the magic cookie in the 
	 * shared classes cache.  If the class is not found in the shared cache or the 
	 * path is not for a class then just use the wrapped bundle file to answer the call. 
	 */
	public BundleEntry getEntry(String path) {
		if (!primed || !path.endsWith(".class"))
			return wrapped.getEntry(path);
		byte[] classbytes = getClassBytes(path.substring(0, path.length()-6));
		if (classbytes == null)
			return wrapped.getEntry(path);
		return new CDSBundleEntry(path, classbytes, wrapped);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.osgi.baseadaptor.bundlefile.BundleFile#getEntryPaths(java.lang.String)
	 * 
	 * Just use the wrapped bundle file to answer the call.
	 */
	public Enumeration getEntryPaths(String path) {
		return wrapped.getEntryPaths(path);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.osgi.baseadaptor.bundlefile.BundleFile#getFile(java.lang.String, boolean)
	 * 
	 * Just use the wrapped bundle file to answer the call.
	 */
	public File getFile(String path, boolean nativeCode) {
		return wrapped.getFile(path, nativeCode);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.osgi.baseadaptor.bundlefile.BundleFile#open()
	 * Just use the wrapped bundle file to answer the call.
	 */
	public void open() throws IOException {
		wrapped.open();
	}

	/**
	 * Returns the file url to the content of the actual bundle file 
	 * @return the file url to the content of the actual bundle file
	 */
	URL getURL() {
		return url;
	}

	/**
	 * Returns the url helper for this bundle file.  This is set by the 
	 * class loading hook
	 * @return the url helper for this bundle file
	 */
	SharedClassURLHelper getURLHelper() {
		return urlHelper;
	}

	/**
	 * Sets the url helper for this bundle file.  This is called by the 
	 * class loading hook.
	 * @param urlHelper the url helper
	 */
	void setURLHelper(SharedClassURLHelper urlHelper) {
		this.urlHelper = urlHelper;
		this.primed = false; // always unprime when a new urlHelper is set
	}

	/**
	 * Sets the primed flag for the bundle file.  This is called by the 
	 * class loading hook after the first class has been loaded from disk for 
	 * this bundle file.
	 * @param primed the primed flag
	 */
	void setPrimed(boolean primed) {
		this.primed = primed;
	}

	/**
	 * Searches in the shared classes cache for the specified class name.
	 * @param name the name of the class
	 * @return the magic cookie to the shared class or null if the class is not in the cache.
	 */
	private byte[] getClassBytes(String name) {
		if (urlHelper == null || url == null)
			return null;
		return urlHelper.findSharedClass(url, name);
	}
}
